/*
	ProWesS, PROGS Windowing System

	ProWesS runtime configuration handling
	PWConfig		configure ProWesS - pass configuration line

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 18 August 1995
	30.01.1996 space after ; and % no longer required
	04.12.1997 test for version now possible (using 'v')
*/

#include <err.h>
#include <mem.h>
#include <job.h>
#include <module.h>
#include <atomic.h>

#include "ProWesS.h"
#include "PWTypeDef.h"
#include "PWConfig.h"

extern Job ProWesS; 				/* job id for myself */

Error PWDefine(char *name, char *value);

/* here come the global variables */
PWTypeDef *xtyp_list=NULL;	/* list of known external types */
#define MAX_PATH 100
char searchpath[MAX_PATH];	/* current searchpath for external types */

static Error LinkNewType(PWTypeDef *curr)
{
	PWTypeDef *list=xtyp_list;
	xtyp_list=curr;
	while(curr->flag!=PW_TYPEDEF_FLAG)
		curr=(PWTypeDef *)curr->flag;
	curr->flag=(int)list;
}

Error PWConfig(char *line)
{
	/* just skip empty and comment lines */
	if (line[0]=='\0' || line[0]==';' || line[0]=='%') return ERR_OK;

	if (line[1]==' ')
	{
		/* command for configurator */
		switch(line[0])
		{
		case 'S':
			/* default search path */
			STRCopySafe(line+2,searchpath,MAX_PATH);
			break;
		case 'T':
			/* load external module with type */
			{
				Error err;
				PWTypeDef *new;

				err=MODULELoadFor(&line[2],searchpath,ProWesS,PW_TYPEDEF_MODULE,&new);
				if (err==ERR_ITNF)
				{
					err=MODULELoadFor(&line[2],PATH_PROGDATA,ProWesS,PW_TYPEDEF_MODULE,&new);
				}
				if (err) return err;

				/* add the new types to the list */
				ATOMICCall(LinkNewType,1,new);
			}
			break;
		case '?':
			/* figure out whether the given type is loaded */
			/* and return ERR_ITNF if it is not */
			{
				PWTypeDef *curr;
				unsigned int type;
				unsigned char name[4];

				MEMCopy(4,&line[2],(char *)name);
				if (name[1]=='\0') name[1]=name[2]=name[3]=' ';
				if (name[2]=='\0') name[2]=name[3]=' ';
				if (name[3]=='\0') name[3]=' ';
				type=((((name[0]*256)+name[1])*256+name[2])*256)+name[3];

				for ( curr=xtyp_list ;
					  curr && curr->identifier!=type ;
					  curr=(PWTypeDef *)curr->flag );
				if (!curr) return ERR_ITNF;
			}
			break;
		case 'v':
			/* check for a minimum version */
			/* gives ERR_ISYN if version is older then requested */
			/* otherwise returns ERR_OK */
			{
				char name[JOB_MAXNAME];
				char *version, *requested=line+2;
				MEMClear(JOB_MAXNAME,name); /* to cater for old syslibs */
				JOBInfo(ProWesS, name, NULL, NULL);
				version=name+strlen(name)+2;
				while (*requested && *requested==*version)
					requested++, version++;
				if (*requested>*version) return ERR_ISYN;
				return ERR_OK;
			}
			break;
		default:
			return ERR_IPAR;
		}
	}
	else
	{
		/* full blown define - has to be passed to all types */
		char *first, *second;
		PWTypeDef *curr;

		/* now this line has to be split in keyword/data */
		second=first=line;
		while (*second && *second!=' ' && *second!='\t') second++;
		if (*second==' ' || *second=='\t')
		{
			*second='\0';
			second++;
		}
		while (*second==' ' || *second=='\t') second++;

		/* and passed to the system and all the objects */
		PWDefine(first,second);
		for (curr=xtyp_list; curr; curr=(PWTypeDef *)curr->flag)
		{
			if (curr->PWDefine)
				curr->PWDefine(first,second);
		}
	}
	return ERR_OK;
}
