/*
	ProWesS, PROGS Windowing System

	PWRedraw

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 14 January 1995
	26.10.1995 PWRedraw is now done in a loop - to make dependencies safe
	24.03.1996 integrated code to call PFPageShow here (less duplication)
*/

#include "private.h"
#include "PWHandler.h"

#define catch(X) do { if (err=(X)) return err; } while (0)

/* external definitions */
extern ColourRGB system_paper;

/* forward declaration */
static Error PWRedrawRegion(PWRegion *region, int which);

#define REDRAW_NONE 0
#define REDRAW_SOME 1
#define REDRAW_ALL	2

Error PWRedraw(PWSystem *system)
{
	PWRegion *region;
	Error err=ERR_OK;
	int which;

	which=REDRAW_NONE;
	if (system->status&SSTATUS_REDRAW) which=REDRAW_SOME;
	if (system->status&SSTATUS_ALLDRAW) which=REDRAW_ALL;
	if (which) system->status|=SSTATUS_PAGESHOW;

	while (which!=REDRAW_NONE)
	{
		system->status&=~(SSTATUS_REDRAW|SSTATUS_ALLDRAW);

		if (which==REDRAW_ALL)
		{
			PFPaperColourRGB(system->gstate,&system_paper);
			PFPageClear(system->gstate);
		}
		for (region=system->regions; region; region=region->next)
		{
			err=PWRedrawRegion(region,which);
			if (err) break;
		}

		which=REDRAW_NONE;
		if (system->status&SSTATUS_REDRAW) which=REDRAW_SOME;
		if (system->status&SSTATUS_ALLDRAW) which=REDRAW_ALL;
	}
	if (system->status&SSTATUS_PAGESHOW)
	{
		PFPageShow(system->gstate);
		system->status&=~SSTATUS_PAGESHOW;
	}
	return err;
}

static Error PWRedrawRegion(PWRegion *region, int which)
{
	Error err=ERR_OK;
	PWObject owner=region->owner;

	if (owner)
	{
		if (owner->status&OSTATUS_REDRAW)
		{
			which=REDRAW_ALL;
			owner->status&=~OSTATUS_REDRAW;
		}

		/* redraw this region ?? */
		if (which==REDRAW_ALL && owner->type->PWDraw)
			catch( owner->type->PWDraw(owner) );
	}

	/* draw all children */
	for (region=region->children; region; region=region->next)
		catch( PWRedrawRegion(region,which) );

	return err;
}
